package main;

import meerkat.crypto.mixnet.Mix2ZeroKnowledgeVerifier;
import meerkat.crypto.mixnet.Mixer;
import meerkat.crypto.mixnet.MixerOutput;
import meerkat.protobuf.BulletinBoardAPI;
import meerkat.protobuf.Crypto;
import necessary.AsyncBulletinBoardClient;


import java.util.ArrayList;
import java.util.List;


/**
 * Created by Tzlil on 12/17/2015.
 */
public class MainMixing {

    private final Mixer mixer;
    private final Mix2ZeroKnowledgeVerifier verifier;
    private final int n, layers;
    private final AsyncBulletinBoardClient asyncBulletinBoardClient;
    private final byte[] id;
    private final int mixerOrder;


    public MainMixing(Mixer mixer, int mixerOrder, Mix2ZeroKnowledgeVerifier verifier, int n
            , AsyncBulletinBoardClient asyncBulletinBoardClient, byte[] id) {
        this.mixer = mixer;
        this.mixerOrder = mixerOrder;
        this.verifier = verifier;
        this.n = n;
        this.layers = (int) (2 * Math.log(n) / Math.log(2)) - 1;  // layers = 2logn -1
        this.asyncBulletinBoardClient = asyncBulletinBoardClient;
        this.id = id;
    }

    public void main(List<Integer> prevBatchIds, int batchId, AsyncBulletinBoardClient.ClientCallback<?> callback) throws Throwable {

        List<Crypto.RerandomizableEncryptedMessage> mixerInput;

        List<BatchHandler> batchHandlers = new ArrayList<BatchHandler>(prevBatchIds.size());
        BatchHandler currentBatchHandler;
        for (Integer prevBatchId : prevBatchIds) {
            currentBatchHandler = new BatchHandler(n, layers,verifier);
            asyncBulletinBoardClient.readBatch(id, prevBatchId,currentBatchHandler);
            batchHandlers.add(currentBatchHandler);
        }

        //  ToDo : assert table continues
        //  ToDo : assert signature validity

        boolean allDone = false;
        while (!allDone) {
            try {
                Thread.sleep(30);
            } catch (InterruptedException e) {
                // do nothing
            }
            // check all handlers messages were received
            allDone = true;
            for (BatchHandler batchHandler : batchHandlers) {
                allDone &= batchHandler.isMsgReceived();
            }
        }

        // assert all handlers succeeded
        for (BatchHandler batchHandler : batchHandlers) {
            if(!batchHandler.verifyTable()){
                throw new Exception("invalid input");
            }
        }

        BatchHandler lastBatchHandler = batchHandlers.get(batchHandlers.size() - 1);
        mixerInput = lastBatchHandler.getInputForMixer();

        MixerOutput mixerOutput = mixer.mix(mixerInput);
        updateBB(mixerOutput, batchId, callback);

    }


    private void updateBB(MixerOutput mixerOutput
            , int batchId, AsyncBulletinBoardClient.ClientCallback<?> callback) {

        BatchConverter batchConverter = new BatchConverter(n,layers);
        List<BulletinBoardAPI.BatchData> batchDataList = batchConverter.MixerOutput2BatchData(mixerOutput);
        asyncBulletinBoardClient.postBatch(id, batchId, batchDataList, callback);
    }




}
