package mixer;

import meerkat.protobuf.Crypto;
import meerkat.protobuf.Mixing;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;


/**
 * Created by Tzlil on 1/18/2016.
 */
public class MixerOutput implements meerkat.crypto.mixnet.MixerOutput{
    private final Mixing.ZeroKnowledgeProof[][] proofs;
    private final Crypto.RerandomizableEncryptedMessage[][] encryptedMessages;
    private final int n;
    private final int layers;

    public MixerOutput(int n,int layers,Mixing.ZeroKnowledgeProof[][] proofs, Crypto.RerandomizableEncryptedMessage[][] encryptedMessages) {
        this.proofs = proofs;
        this.encryptedMessages = encryptedMessages;
        this.n = n;
        this.layers = layers;
    }

    public MixerOutput(meerkat.crypto.mixnet.MixerOutput mixerOutput) {
        this.proofs = mixerOutput.getProofs();
        this.encryptedMessages = mixerOutput.getEncryptedMessages();
        this.n = mixerOutput.getN();
        this.layers = (int) (2 * Math.log(n) / Math.log(2)) - 1;  // layers = 2logn -1;
    }


    @Override
    public Mixing.ZeroKnowledgeProof[][] getProofs() {
        return proofs;
    }

    @Override
    public Crypto.RerandomizableEncryptedMessage[][] getEncryptedMessages() {
        return encryptedMessages;
    }

    @Override
    public int getN() {
        return n;
    }

    public void outToFile(String dir) throws IOException {

        (new File(dir)).mkdirs();
        //create files
        String proofsDir = dir + "/Proofs";
        String encDir = dir + "/EncryptedMessages";
        (new File(proofsDir)).mkdir();
        (new File(encDir)).mkdir();
        for (int layer = 0; layer < layers; layer++){
            (new File(proofsDir +"/layer" + layer )).mkdir();
            (new File(encDir +"/layer" + layer )).mkdir();
        }
        (new File(encDir +"/input")).mkdir();


        for (int layer = 0; layer < layers; layer++){
            for(int i = 0; i < proofs[layer].length; i ++){
                writeProofToFile(proofsDir,proofs[layer][i]);
            }
        }

        for (int layer = 0; layer <= layers; layer++){
            for(int i = 0; i < encryptedMessages[layer].length; i ++){
                writeEncToFile(encDir,layer - 1, i,encryptedMessages[layer][i]);
            }
        }
    }

    private void writeProofToFile(String proofsDir, Mixing.ZeroKnowledgeProof proof) throws IOException {
        Mixing.ZeroKnowledgeProof.Location location = proof.getLocation();
        int layer = location.getLayer();
        int i = location.getI();
        int j = location.getJ();
        String fileName = proofsDir+"/layer" + layer +"/" + i +"_" + j;

        File file = new File(fileName);
        file.createNewFile();
        FileOutputStream fos = new FileOutputStream(file);
        fos.write(proof.toByteArray());
        fos.close();
    }

    private void writeEncToFile(String encDir,int layer,int i, Crypto.RerandomizableEncryptedMessage enc) throws IOException {

        String fileName;
        if(layer >= 0)
            fileName = encDir+"/layer" + layer +"/" + i;
        else
            fileName =  encDir+"/input/" + i;

        File file = new File(fileName);
        file.createNewFile();
        FileOutputStream fos = new FileOutputStream(file);
        fos.write(enc.toByteArray());
        fos.close();
    }
}
