package mixer;

/**
 * Created by Tzlil on 12/30/2015.
 */

import com.google.protobuf.InvalidProtocolBufferException;
import meerkat.crypto.concrete.ECElGamalEncryption;
import meerkat.crypto.mixnet.*;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.Mixing;
import meerkat.protobuf.Voting;
import org.junit.Before;
import org.junit.Test;
import prover.Prover;
import qilin.primitives.RandomOracle;
import qilin.primitives.concrete.DigestOracle;
import qilin.primitives.concrete.ECElGamal;
import qilin.primitives.concrete.ECGroup;
import qilin.util.Pair;
import verifier.Verifier;
import verifier.VerifyTable;
import java.security.spec.InvalidKeySpecException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class MixingText {

    ECElGamalEncryption encryptor;
    ECGroup group;
    Random random,randomMixer,randomProver;
    RandomOracle randomOracle;
    Mix2ZeroKnowledgeVerifier verifier;
    Prover prover;
    meerkat.crypto.mixnet.Mixer mixer;
    private int layers;
    private int n;


    @Before
    public void setup() throws InvalidKeySpecException {
        // initialization
        random = new Random();
        group = new ECGroup("secp256k1");
        encryptor = new ECElGamalEncryption();
        encryptor.init(Utiles.serializePk(group, new ECElGamal.SK(group, ECElGamal.generateSecretKey(group, random))));
        randomMixer = new Random();
        randomProver = new Random();
        randomOracle = new DigestOracle();
        verifier = new Verifier(encryptor,randomOracle);
        prover = new Prover(randomProver,encryptor,randomOracle);
        mixer = new Mixer(randomMixer,prover,encryptor);

        // generate n
        int logN = 8; //  + random.nextInt(8)
        layers = 2*logN - 1;
        n = 1 << logN;
    }

    public List<Crypto.RerandomizableEncryptedMessage> generateMixerInput(){
        List<Crypto.RerandomizableEncryptedMessage> result = new ArrayList<Crypto.RerandomizableEncryptedMessage>();
        Voting.PlaintextBallot msg;
        for (int i = 0; i < n ; i++){
            msg = Utiles.genRandomBallot(2,3,16);
            result.add(encryptor.encrypt(msg, encryptor.generateRandomness(random)));
        }
        return result;
    }

    @Test
    public void mixingTest() throws InvalidProtocolBufferException {
        System.out.println("n is : " + n);
        System.out.println("  generating input");
        List<Crypto.RerandomizableEncryptedMessage> mixerInput = generateMixerInput();
        System.out.println("  start mixing");
        long startTime = System.currentTimeMillis();

        meerkat.crypto.mixnet.MixerOutput mixerOutput = mixer.mix(mixerInput);

        long finishTime = System.currentTimeMillis();
        System.out.println("    that took: "+(finishTime-startTime)+ " ms");

        System.out.println("start verification");
        startTime = System.currentTimeMillis();

        assert (VerifyTable.verifyTable(verifier,n,mixerOutput));

        finishTime = System.currentTimeMillis();
        System.out.println("    that took: "+(finishTime-startTime)+ " ms");
    }
}
